/*

                          Argonne National Laboratory
                            APS Operations Division
                     Beamline Controls and Data Acquisition

                              EPICS Brick Athena



 -----------------------------------------------------------------------------
                                COPYRIGHT NOTICE
 -----------------------------------------------------------------------------
   Copyright (c) 2002 The University of Chicago, as Operator of Argonne
      National Laboratory.
   Copyright (c) 2002 The Regents of the University of California, as
      Operator of Los Alamos National Laboratory.
   Synapps Versions 4-5
   and higher are distributed subject to a Software License Agreement found
   in file LICENSE that is included with this distribution.
 -----------------------------------------------------------------------------

 Description
    This module provides the initialization methods for the Athena SBC.

        drvAthenaInit(rstbrd,rstda,rstfifo,clrdma,clrt,clrd,clra)

        Where:
            rst* - reset
            clr* - clear interrupt

 Developer notes:
    1. A value of 1 indicates to perform the reset or clear.
    2. A value of 0 indicates not to perform the reset or clear.

 Source control info:
    Modified by:    $Author: dkline $
                    $Date: 2006/05/15 15:31:20 $
                    $Revision: 1.1 $

 =============================================================================
 History:
 Author: David M. Kline
 -----------------------------------------------------------------------------
 2006-May-15  DMK  Derived from drvDac source.
 -----------------------------------------------------------------------------

*/


/* EPICS base version-specific definitions (must be performed first) */
#include <epicsVersion.h>
#define LT_EPICSBASE(v,r,l) (EPICS_VERSION<(v)||(EPICS_VERSION==(v)&&(EPICS_REVISION<(r)||(EPICS_REVISION==(r)&&EPICS_MODIFICATION<(l)))))


/* Evaluate EPICS base */
#if LT_EPICSBASE(3,14,6)
    #error "EPICS base must be 3.14.6 or greater"
#endif

/* System related include files */
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


/* EPICS system related include files */
#include <iocsh.h>
#include <epicsStdio.h>
#include <cantProceed.h>
#include <epicsString.h>
#include <epicsExport.h>


/* DSC related include files */
#include "dscsup.h"


/* Define symbolic constants */
#define TRUE       (1)
#define FALSE      (0)


/* Public interface forward references */
static int drvAthenaInit(int rstbrd,int rstda,int rstfifo,int clrdma,int clrt,int clrd,int clra);


/* Define macros */
#define ISOK(s) (asynSuccess==(s))
#define ISNOTOK(s) (!ISOK(s))


/****************************************************************************
 * Define public interface methods
 ****************************************************************************/
static int drvAthenaInit(int rstbrd,int rstda,int rstfifo,int clrdma,int clrt,int clrd,int clra)
{
    unsigned char cmd = 0;

    if( rstbrd )
        cmd |= ATH_CMD_RSTBRD;
    if( rstda )
        cmd |= ATH_CMD_RSTDA;
    if( rstfifo )
        cmd |= ATH_CMD_RSTFIFO;
    if( clrdma )
        cmd |= ATH_CMD_CLRDMA;
    if( clrt )
        cmd |= ATH_CMD_CLRT;
    if( clrd )
        cmd |= ATH_CMD_CLRD;
    if( clra )
        cmd |= ATH_CMD_CLRA;

    putdaq(ATH_DAQ_BASE,ATH_OUTB_COMMAND,cmd);

    return( 0 );
}


/****************************************************************************
 * Register public methods
 ****************************************************************************/

/* Initialization method definitions */
static const iocshArg drvAthenaInitArg0 = {"rstbrd",iocshArgInt};
static const iocshArg drvAthenaInitArg1 = {"rstda",iocshArgInt};
static const iocshArg drvAthenaInitArg2 = {"rstfifo",iocshArgInt};
static const iocshArg drvAthenaInitArg3 = {"clrdma",iocshArgInt};
static const iocshArg drvAthenaInitArg4 = {"clrt",iocshArgInt};
static const iocshArg drvAthenaInitArg5 = {"clrd",iocshArgInt};
static const iocshArg drvAthenaInitArg6 = {"clra",iocshArgInt};
static const iocshArg* drvAthenaInitArgs[]= {&drvAthenaInitArg0,&drvAthenaInitArg1,&drvAthenaInitArg2,&drvAthenaInitArg3,&drvAthenaInitArg4,&drvAthenaInitArg5,&drvAthenaInitArg6};
static const iocshFuncDef drvAthenaInitFuncDef = {"drvAthenaInit",7,drvAthenaInitArgs};
static void drvAthenaInitCallFunc(const iocshArgBuf* args)
{
    drvAthenaInit(args[0].ival,args[1].ival,args[2].ival,args[3].ival,args[4].ival,args[5].ival,args[6].ival);
}

/* Registration method */
static void drvAthenaRegister(void)
{
    static int firstTime = 1;

    if( firstTime )
    {
        firstTime = 0;
        iocshRegister( &drvAthenaInitFuncDef, drvAthenaInitCallFunc );
    }
}
epicsExportRegistrar( drvAthenaRegister );
