#ifndef _sensuph
#define _sensuph
/*

                          Argonne National Laboratory
                            APS Operations Division
                     Beamline Controls and Data Acquisition

                   Support for Sensoray's Smart A/D Model 518



 -----------------------------------------------------------------------------
                                COPYRIGHT NOTICE
 -----------------------------------------------------------------------------
   Copyright (c) 2002 The University of Chicago, as Operator of Argonne
      National Laboratory.
   Copyright (c) 2002 The Regents of the University of California, as
      Operator of Los Alamos National Laboratory.
   Synapps Versions 4-5
   and higher are distributed subject to a Software License Agreement found
   in file LICENSE that is included with this distribution.
 -----------------------------------------------------------------------------

 Description
    This module provides the low-level support for Sensoray's PC104-based
    Smart A/D Model 518. 

 Developer notes:

 Source control info:
    Modified by:    $Author: dkline $
                    $Date: 2006/06/21 19:19:25 $
                    $Revision: 1.4 $

 =============================================================================
 History:
 Author: David M. Kline
 -----------------------------------------------------------------------------
 2006-Mar-09  DMK  Development begins.
 2006-May-15  DMK  Removed senInitBoard().
 -----------------------------------------------------------------------------

*/


/* System related include files */
#include <sys/io.h>


/* Board definition include files */
#include "boardtypes.h"


/******************************************************************************
 * Define symbolic constants
 ******************************************************************************/

/* Define misc. symbolic constants */
#define SEN_CHNS        (8)

/* Define IO register map symbolic constants */
#define SEN_INPB_DAT    (0)
#define SEN_INPB_STS    (1)

#define SEN_OUTB_CMD    (0)
#define SEN_OUTB_CTL    (1)

/* Define status register symbolic constants */
#define SEN_STS_CRMT    (0x80)
#define SEN_STS_DAV     (0x40)
#define SEN_STS_ALARM   (0x20)
#define SEN_STS_FAULT   (0x10)

/* Define control register symbolic constants */
#define SEN_CTL_INT     (0x80)
#define SEN_CTL_ENA     (0x10)
#define SEN_CTL_ICMD    (0x04)
#define SEN_CTL_IDAT    (0x02)
#define SEN_CTL_IALM    (0x01)


/******************************************************************************
 * Define access macros
 ******************************************************************************/
#define getsen(b,o) inb((b)+(o))
#define putsen(b,o,v) outb((v),(b)+(o))


#ifdef __cplusplus
extern "C" {
#endif

extern char senReadB(unsigned short base);
extern void senWriteB(unsigned short base,unsigned char dat);
extern short senReadW(unsigned short base);
extern void senWriteW(unsigned short base,short dat);

extern void senReset(unsigned short base);
extern short senGetStatus(unsigned short base);
extern float senGetTemp(unsigned short base);
extern float senGetVersion(unsigned short base);
extern short senGetProductID(unsigned short base);
extern short senGetAlarmFlags(unsigned short base);
extern short senGetChan(unsigned short base,short chn);
extern void senGetChans(unsigned short base,short* dat);

extern short senGetType(short typ);
extern float senGetScalar(short typ);

extern void senSetSpeed(unsigned short base);
extern void senSetType(unsigned short base,short chn,short typ);

#ifdef __cplusplus
}
#endif

#endif /* _dscsuph */
